@@ -2,20 +2,26 @@ use crate::server::{Accept, Handle, ListenerTask, MakeParts};
 use crate::util::HyperService;
 
 use std::io;
-use std::net::SocketAddr;
+use std::pin::Pin;
+use std::task::Poll;
 
 use futures_util::future::Ready;
+use futures_util::stream::{Stream, StreamExt, FuturesUnordered};
 
 use tokio::io::{AsyncRead, AsyncWrite};
 use tokio::net::TcpListener;
 use tokio::task::JoinHandle;
 
+use http::uri::Scheme;
+
 use tower_http::add_extension::AddExtension;
 use tower_layer::Layer;
 
 use hyper::server::conn::Http;
 use hyper::Request;
 
+type FutList = FuturesUnordered<JoinHandle<()>>;
+
 #[derive(Clone)]
 pub(crate) struct CloneParts<L, A> {
     layer: L,
@@ -52,11 +58,13 @@ pub(crate) struct HttpServer<S, M> {
     service: S,
     handle: Handle,
     make_parts: M,
+    scheme: Scheme,
 }
 
 impl<S, M> HttpServer<S, M> {
-    pub(crate) fn new(service: S, handle: Handle, make_parts: M) -> Self {
+    pub(crate) fn new(scheme: Scheme, service: S, handle: Handle, make_parts: M) -> Self {
         Self {
+            scheme,
             service,
             handle,
             make_parts,
@@ -65,50 +73,72 @@ impl<S, M> HttpServer<S, M> {
 }
 
 impl<S> HttpServer<S, CloneParts<NoopLayer, NoopAcceptor>> {
-    pub(crate) fn from_service(service: S, handle: Handle) -> Self {
-        HttpServer::new(service, handle, CloneParts::new(NoopLayer, NoopAcceptor))
+    pub(crate) fn from_service(scheme: Scheme, service: S, handle: Handle) -> Self {
+        HttpServer::new(
+            scheme,
+            service,
+            handle,
+            CloneParts::new(NoopLayer, NoopAcceptor),
+        )
     }
 }
 
 #[cfg(feature = "tls-rustls")]
 impl<S, A> HttpServer<S, CloneParts<NoopLayer, A>> {
-    pub(crate) fn from_acceptor(service: S, handle: Handle, acceptor: A) -> Self {
-        HttpServer::new(service, handle, CloneParts::new(NoopLayer, acceptor))
+    pub(crate) fn from_acceptor(scheme: Scheme, service: S, handle: Handle, acceptor: A) -> Self {
+        HttpServer::new(
+            scheme,
+            service,
+            handle,
+            CloneParts::new(NoopLayer, acceptor),
+        )
     }
 }
 
 macro_rules! accept {
-    ($handle:expr, $listener:expr) => {
+    ($conns:expr, $handle:expr, $listener:expr) => {
         tokio::select! {
             biased;
             _ = $handle.shutdown_signal() => break Mode::Shutdown,
             _ = $handle.graceful_shutdown_signal() => break Mode::Graceful,
             result = $listener.accept() => result,
+            _ = poll_list(&mut $conns) => unreachable!(),
         }
     };
 }
 
+macro_rules! clear_finished {
+    ($conns:expr) => {
+        futures_util::future::poll_fn(|cx| {
+            while let Poll::Ready(_) = Pin::new(&mut $conns).poll_next(cx) {}
+
+            Poll::Ready(())
+        }).await;
+    }
+}
+
 impl<S, M> HttpServer<S, M>
 where
     S: HyperService<Request<hyper::Body>>,
     M: MakeParts + Clone + Send + Sync + 'static,
-    M::Layer: Layer<AddExtension<S, SocketAddr>> + Clone + Send + Sync + 'static,
-    <M::Layer as Layer<AddExtension<S, SocketAddr>>>::Service: HyperService<Request<hyper::Body>>,
+    M::Layer: Layer<S> + Clone + Send + Sync + 'static,
+    <M::Layer as Layer<S>>::Service: HyperService<Request<hyper::Body>>,
     M::Acceptor: Accept,
 {
     pub(crate) fn serve_on(&self, listener: TcpListener) -> ListenerTask {
         let server = self.clone();
 
         tokio::spawn(async move {
-            let mut conns = Vec::new();
+            let mut conns = FuturesUnordered::new();
 
             let mode = loop {
-                let (stream, addr) = accept!(&server.handle, &listener)?;
+                let (stream, addr) = accept!(conns, server.handle, listener)?;
                 let (layer, acceptor) = server.make_parts.make_parts();
 
                 let service = server.service.clone();
-                let service = AddExtension::new(service, addr);
                 let service = layer.layer(service);
+                let service = AddExtension::new(service, addr);
+                let service = AddExtension::new(service, server.scheme.clone());
 
                 let conn = tokio::spawn(async move {
                     if let Ok(stream) = acceptor.accept(stream).await {
@@ -120,6 +150,8 @@ where
                 });
 
                 conns.push(conn);
+
+                clear_finished!(conns);
             };
 
             drop(listener);
@@ -138,16 +170,18 @@ where
     }
 }
 
-fn shutdown_conns(conns: Vec<JoinHandle<()>>) {
-    for conn in conns {
-        conn.abort();
-    }
+async fn poll_list(conns: &mut FutList) {
+    while let Some(_) = conns.next().await {}
+
+    std::future::pending::<()>().await;
 }
 
-async fn wait_conns(conns: &mut Vec<JoinHandle<()>>) {
-    for conn in conns {
-        let _ = conn.await;
-    }
+fn shutdown_conns(conns: FutList) {
+    conns.iter().for_each(|conn| conn.abort());
+}
+
+async fn wait_conns(conns: &mut FutList) {
+    while let Some(_) = conns.next().await {}
 }
 
 #[derive(Clone)]
