@@ -34,14 +34,14 @@ use tokio::net::{TcpListener, TcpStream};
 use tokio::sync::Notify;
 use tokio::task::{spawn_blocking, JoinHandle};
 
-use tower_http::add_extension::AddExtension;
-use tower_layer::Layer;
-use tower_service::Service;
-
 use http::request::Request;
 use http::response::Response;
+use http::uri::Scheme;
 use http_body::Body;
 
+use tower_layer::Layer;
+use tower_service::Service;
+
 type ListenerTask = JoinHandle<io::Result<()>>;
 type FutList = FuturesUnordered<ListenerTask>;
 
@@ -179,8 +179,10 @@ impl Server {
         B::Data: Send,
         B::Error: Into<Box<dyn std::error::Error + Send + Sync>>,
     {
-        self.custom_serve(move |handle| HttpServer::from_service(service.clone(), handle))
-            .await
+        self.custom_serve(move |handle| {
+            HttpServer::from_service(Scheme::HTTP, service.clone(), handle)
+        })
+        .await
     }
 
     /// Serve provided cloneable service on all binded addresses.
@@ -204,8 +206,10 @@ impl Server {
         B::Data: Send,
         B::Error: Into<Box<dyn std::error::Error + Send + Sync>>,
     {
-        self.custom_serve(move |handle| HttpServer::recording_from_service(service.clone(), handle))
-            .await
+        self.custom_serve(move |handle| {
+            HttpServer::recording_from_service(Scheme::HTTP, service.clone(), handle)
+        })
+        .await
     }
 
     async fn custom_serve<F, S, M>(self, make_server: F) -> io::Result<()>
@@ -213,9 +217,8 @@ impl Server {
         F: Fn(Handle) -> HttpServer<S, M>,
         S: HyperService<Request<hyper::Body>>,
         M: MakeParts + Clone + Send + Sync + 'static,
-        M::Layer: Layer<AddExtension<S, SocketAddr>> + Clone + Send + Sync + 'static,
-        <M::Layer as Layer<AddExtension<S, SocketAddr>>>::Service:
-            HyperService<Request<hyper::Body>>,
+        M::Layer: Layer<S> + Clone + Send + Sync + 'static,
+        <M::Layer as Layer<S>>::Service: HyperService<Request<hyper::Body>>,
         M::Acceptor: Accept,
     {
         serve(move |mut fut_list| async move {
@@ -395,8 +398,8 @@ where
     F: Fn(Handle) -> HttpServer<S, M>,
     S: HyperService<Request<hyper::Body>>,
     M: MakeParts + Clone + Send + Sync + 'static,
-    M::Layer: Layer<AddExtension<S, SocketAddr>> + Clone + Send + Sync + 'static,
-    <M::Layer as Layer<AddExtension<S, SocketAddr>>>::Service: HyperService<Request<hyper::Body>>,
+    M::Layer: Layer<S> + Clone + Send + Sync + 'static,
+    <M::Layer as Layer<S>>::Service: HyperService<Request<hyper::Body>>,
     M::Acceptor: Accept,
 {
     let http_server = make_server(handle.clone());
